/* mountlist.c -- return a list of mounted filesystems
   Copyright (C) 1991, 1992 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include "mountlist.h"
#include "system.h"
#include <stdlib.h>
#include <string.h>

#if defined (MOUNTED_GETFSSTAT)	/* __alpha running OSF_1 */
#  include <sys/mount.h>
#  include <sys/fs_types.h>
#endif				/* MOUNTED_GETFSSTAT */

#ifdef MOUNTED_GETMNTENT1	/* 4.3BSD, SunOS, HP-UX, Dynix, Irix.  */
#include <mntent.h>
#if !defined(MOUNTED)
#  if defined(MNT_MNTTAB)	/* HP-UX.  */
#    define MOUNTED MNT_MNTTAB
#  endif
#  if defined(MNTTABNAME)	/* Dynix.  */
#    define MOUNTED MNTTABNAME
#  endif
#endif
#endif

/* Return the value of the hexadecimal number represented by CP.
   No prefix (like '0x') or suffix (like 'h') is expected to be
   part of CP. */

static int xatoi(cp)
char *cp;
{
	int val;

	val = 0;
	while (*cp) {
		if (*cp >= 'a' && *cp <= 'f')
			val = val * 16 + *cp - 'a' + 10;
		else if (*cp >= 'A' && *cp <= 'F')
			val = val * 16 + *cp - 'A' + 10;
		else if (*cp >= '0' && *cp <= '9')
			val = val * 16 + *cp - '0';
		else
			break;
		cp++;
	}
	return val;
}


/* Return a list of the currently mounted filesystems, or NULL on error.
   Add each entry to the tail of the list so that they stay in order.
   If NEED_FS_TYPE is nonzero, ensure that the filesystem type fields in
   the returned list are valid.  Otherwise, they might not be.
   If ALL_FS is zero, do not return entries for filesystems that
   are automounter (dummy) entries.  */

struct mount_entry *read_filesystem_list(need_fs_type, all_fs)
int need_fs_type, all_fs;
{
	struct mount_entry *mount_list;
	struct mount_entry *me;
	struct mount_entry *mtail;

	/* Start the list off with a dummy entry. */
	me = (struct mount_entry *) xmalloc(sizeof(struct mount_entry));
	me->me_next = NULL;
	mount_list = mtail = me;

	{
		struct mntent *mnt;
		char *table = MOUNTED;
		FILE *fp;
		char *devopt;

		fp = setmntent(table, "r");
		if (fp == NULL)
			return NULL;

		while ((mnt = getmntent(fp))) {
			if (!all_fs && (!strcmp(mnt->mnt_type, "ignore")
					|| !strcmp(mnt->mnt_type, "auto")))
				continue;

			me = (struct mount_entry *)
			    xmalloc(sizeof(struct mount_entry));
			me->me_devname = xstrdup(mnt->mnt_fsname);
			me->me_mountdir = xstrdup(mnt->mnt_dir);
			me->me_type = xstrdup(mnt->mnt_type);
			devopt = strstr(mnt->mnt_opts, "dev=");
			if (devopt) {
				if (devopt[4] == '0'
				    && (devopt[5] == 'x'
					|| devopt[5] == 'X'))
					me->me_dev = xatoi(devopt + 6);
				else
					me->me_dev = xatoi(devopt + 4);
			} else
				me->me_dev = -1;	/* Magic; means not known yet. */
			me->me_next = NULL;

			/* Add to the linked list. */
			mtail->me_next = me;
			mtail = me;
		}

		if (endmntent(fp) == 0)
			return NULL;
	}


	/* Free the dummy head. */
	me = mount_list;
	mount_list = mount_list->me_next;
	free(me);
	return mount_list;
}
